from django.utils.functional import LazyObject
from emailsys.helpers import get_module_class
from emailsys import settings

class WebService(LazyObject):
    def _setup(self):
        self._wrapped = get_module_class(settings.WEBSERVICE)()

class Sender(LazyObject):
    def _setup(self):
        self._wrapped = get_module_class(settings.SENDER)() 

webservice = WebService()
sender = Sender()


from django.contrib import admin
from emailsys.models import EmailLog

class EmailMessageAdmin(admin.ModelAdmin):
    list_display = ('message_id', 'time', 'to_email' ,'command', 'error' )
    list_filter = ['message_id', 'to_email', 'command',]

admin.site.register(EmailLog, EmailMessageAdmin)


from celery import task
from emailsys import default
from emailsys.models import Message, SmsCampaign

@task
def webservice_send_mail(subject, emails, content, command):
    """
    apply default.sender inside the task
    """

    msg = Message(subject)
    msg.emails = emails
    msg.content = content
    msg.command = command
    default.webservice.SendMessageToEmails(msg)

@task
def webservice_send_sms(to_numbers, content):
    """
    apply default.sender inside the task
    """

    sms = SmsCampaign(content)
    sms.to_numbers = to_numbers
    sms.content = content
    default.webservice.SendSMSCampaignToMobiles(sms)



from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy  as _
from emailsys.settings import FROM_ADDRESS, FROM_NAME, REPLAY_TO, CONTENT_TYPE, SMS_FROM_NUMBER,\
    SMS_FROM_NAME
from emailsys.helpers import get_message_template

class EmailLog(models.Model):
    message_id = models.CharField(_('Message Id'), max_length=10)
    to_email = models.EmailField(_('To Email'), max_length=254)
    time = models.DateTimeField(_('Sent At'), default=timezone.now())
    subject = models.CharField(_('Subject'),max_length=150,null=True)
    command = models.CharField(_('Command'),max_length=50)
    response = models.TextField(_('Response'),max_length=50)
    error = models.SmallIntegerField(_('Error'),default=0)
    
    def __unicode__(self):
        return self.to_email

    class Meta:
        verbose_name = _('Email Logs')
        verbose_name_plural = _('Email Logs')


'''
Message and SMS object (not models)
'''
class Message(object):
    # settings
    from_addr = FROM_ADDRESS
    from_name = FROM_NAME
    reply_to = REPLAY_TO
    encoding = 'utf-8'
    content_type = CONTENT_TYPE
    
    subject = None
    content = None
    emails = None
    template = None
    command = None
    
    def __init__(self, subject, emails=None, content=None):
        self.subject = subject
        self.content = content
        self.emails = emails

    def __unicode__(self):
        return self.emails

    def add_email(self, _email):
        if self.emails is None:
            self.emails = []
            
        self.emails.append(_email)
    
    def set_command(self, command):
        self.command = command

    def use_template(self, template, context):
        self.content = get_message_template(template, context)
    
class SmsCampaign(object):
    from_number = SMS_FROM_NUMBER
    from_name = SMS_FROM_NAME
    
    content = None
    
    to_numbers = None
    
    def __init__(self, content=None, to_numbers=None):
        self.content = content
        self.to_numbers = to_numbers
        
    def __unicode__(self):
        return self.to_numbers
    
    def add_number(self, _number):
        if self.to_numbers is None:
            self.to_numbers = []
            
        self.to_numbers.append(_number)
        
    def validate(self):
        # need to validate the campaign before sending
        pass



# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

setup(
    name='django-emailsys',
    version='0.0.23',
    author=u'Ron Sneh',
    author_email='me@ronsneh.com',
    packages=find_packages(),
    url='https://bitbucket.org/rsneh/django-emailsys',
    description='Provide the ability to send emails/sms using WebServices',
    long_description=open('README').read(),
    include_package_data=True,
    zip_safe=False,
)



from emailsys import settings, default
from django.core.mail.message import EmailMultiAlternatives
from emailsys.tasks import webservice_send_mail, webservice_send_sms

'''
TODO: Implement BaseSender for basic settings and functionality
''' 
class BaseSender(object):
    pass

class DjangoSender(BaseSender):
    def send(self, Message=None):
        fail_silently = False

        if Message is None:
            raise Exception("Cannot generate message to send")
    
        # Send emails.
        for addr in Message.emails:
            msg = EmailMultiAlternatives(Message.subject, Message.content_txt, Message.from_addr, [addr])
            if msg.content_html is not None:
                msg.attach_alternative(msg.content_html, "text/html")
            
            msg.send(fail_silently=fail_silently)
    pass

class ActivetrailSender(BaseSender):
    def send(self, Message=None):
        
        if Message is None:
            raise Exception("Cannot generate message to send")

        # Check if USE_CELERY
        if settings.USE_CELERY:
            webservice_send_mail.delay(Message.subject, Message.emails, Message.content, Message.command)
        else:
            webservice = default.webservice
            webservice.SendMessageToEmails(Message)
            
    def sms(self, SmsCampaign=None):
        
        if SmsCampaign is None:
            raise Exception("Cannot generate message to send")

        # Check if USE_CELERY
        if settings.USE_CELERY:
            webservice_send_sms.delay(SmsCampaign.to_numbers, SmsCampaign.content)
        else:
            webservice = default.webservice
            webservice.SendSMSCampaignToMobiles(SmsCampaign)